<?php
date_default_timezone_set("Asia/Ho_Chi_Minh");
// Cấu hình kết nối SQLite
class Database
{
    private $db;

    public function __construct()
    {
        try {
            // Kết nối hoặc tạo CSDL SQLite
            $this->db = new SQLite3('public_notes.db');

            // Tạo bảng Users nếu chưa tồn tại
            $this->db->exec('CREATE TABLE IF NOT EXISTS users (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                username TEXT UNIQUE,
                color TEXT,
                is_admin INTEGER DEFAULT 0
            )');

            // Tạo bảng Messages nếu chưa tồn tại
            $this->db->exec('CREATE TABLE IF NOT EXISTS messages (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                user_id INTEGER,
                text TEXT,
                timestamp DATETIME DEFAULT CURRENT_TIMESTAMP,
                FOREIGN KEY(user_id) REFERENCES users(id)
            )');

            // Kiểm tra và tạo admin mặc định nếu chưa tồn tại
            $checkAdmin = $this->db->query('SELECT COUNT(*) as count FROM users WHERE is_admin = 1');
            $adminCount = $checkAdmin->fetchArray(SQLITE3_ASSOC)['count'];

            if ($adminCount == 0) {
                $stmt = $this->db->prepare('INSERT INTO users (username, color, is_admin) VALUES (:username, :color, 1)');
                $stmt->bindValue(':username', 'admin', SQLITE3_TEXT);
                $stmt->bindValue(':color', '#FF0000', SQLITE3_TEXT);
                $stmt->execute();
            }
        } catch (Exception $e) {
            die("Lỗi kết nối CSDL: " . $e->getMessage());
        }
    }

    // Hàm sinh màu ngẫu nhiên
    private function generateColor()
    {
        return sprintf("#%06x", mt_rand(0, 0xFFFFFF));
    }

    // Tạo hoặc lấy user
    public function createOrGetUser($username)
    {
        // Kiểm tra user đã tồn tại chưa
        $stmt = $this->db->prepare('SELECT * FROM users WHERE username = :username');
        $stmt->bindValue(':username', $username, SQLITE3_TEXT);
        $result = $stmt->execute()->fetchArray(SQLITE3_ASSOC);

        if ($result) {
            return $result;
        }

        // Tạo user mới
        $color = $this->generateColor();
        $stmt = $this->db->prepare('INSERT INTO users (username, color) VALUES (:username, :color)');
        $stmt->bindValue(':username', $username, SQLITE3_TEXT);
        $stmt->bindValue(':color', $color, SQLITE3_TEXT);
        $stmt->execute();

        return [
            'id' => $this->db->lastInsertRowID(),
            'username' => $username,
            'color' => $color,
            'is_admin' => 0
        ];
    }

    // Kiểm tra quyền admin
    public function isAdmin($userId)
    {
        $stmt = $this->db->prepare('SELECT is_admin FROM users WHERE id = :user_id');
        $stmt->bindValue(':user_id', $userId, SQLITE3_INTEGER);
        $result = $stmt->execute()->fetchArray(SQLITE3_ASSOC);
        return $result ? $result['is_admin'] == 1 : false;
    }

    // Lưu tin nhắn
    public function saveMessage($userId, $text)
    {
        $stmt = $this->db->prepare('INSERT INTO messages (user_id, text) VALUES (:user_id, :text)');
        $stmt->bindValue(':user_id', $userId, SQLITE3_INTEGER);
        $stmt->bindValue(':text', $text, SQLITE3_TEXT);
        $stmt->execute();
        return $this->db->lastInsertRowID();
    }

    // Lấy tin nhắn với các bộ lọc
    public function getMessages($filters = [])
    {
        $query = 'SELECT m.id, m.text, m.timestamp, u.username, u.color 
                  FROM messages m 
                  JOIN users u ON m.user_id = u.id 
                  WHERE 1=1';

        // Áp dụng bộ lọc
        if (!empty($filters['username'])) {
            $query .= " AND u.username = '{$filters['username']}'";
        }

        if (!empty($filters['start_date'])) {
            $query .= " AND m.timestamp >= '{$filters['start_date']}'";
        }

        if (!empty($filters['end_date'])) {
            $query .= " AND m.timestamp <= '{$filters['end_date']}'";
        }

        $query .= ' ORDER BY m.timestamp DESC';

        $results = $this->db->query($query);
        $messages = [];

        while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
            $messages[] = $row;
        }

        return $messages;
    }

    // Xóa toàn bộ tin nhắn (chỉ admin mới được)
    public function clearMessages($userId)
    {
        if ($this->isAdmin($userId)) {
            $this->db->exec('DELETE FROM messages');
            return true;
        }
        return false;
    }

    // Đóng kết nối CSDL
    public function __destruct()
    {
        $this->db->close();
    }
}

// Bắt đầu session để quản lý trạng thái người dùng
session_start();

// Xử lý các yêu cầu API
// header('Content-Type: application/json');
// header('Access-Control-Allow-Origin: *');
// header('Access-Control-Allow-Methods: GET, POST, OPTIONS');
// header('Access-Control-Allow-Headers: Content-Type');

$db = new Database();

// Xác định hành động
$action = $_GET['action'] ?? $_POST['action'] ?? null;

switch ($action) {
    case 'create_user':
        // Sử dụng cookie hoặc username được cung cấp
        $username = $_POST['username'] ?? $_COOKIE['uname'] ?? '';
        $user = $db->createOrGetUser($username);
        echo json_encode($user);
        break;

    case 'send_message':
        $userId = $_POST['user_id'] ?? 0;
        $text = $_POST['text'] ?? '';
        $messageId = $db->saveMessage($userId, $text);
        echo json_encode(['id' => $messageId, 'text' => $text]);
        break;

    case 'get_messages':
        $filters = [
            'username' => $_GET['username'] ?? null,
            'start_date' => $_GET['start_date'] ?? null,
            'end_date' => $_GET['end_date'] ?? null
        ];
        $messages = $db->getMessages($filters);
        echo json_encode($messages);
        break;

    case 'clear_messages':
        $userId = $_POST['user_id'] ?? 0;
        $result = $db->clearMessages($userId);
        echo json_encode([
            'status' => $result ? 'success' : 'unauthorized',
            'message' => $result ? 'Đã xóa tin nhắn' : 'Bạn không có quyền xóa'
        ]);
        break;

    default:
        // Trả về trang HTML nếu không có action
?>
        <!DOCTYPE html>
        <html>

        <head>
            <title>Ghi Chú Công Cộng</title>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet">
            <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/js/bootstrap.bundle.min.js"></script>
            <script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
            <link href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.7.1/css/all.min.css" rel="stylesheet">

            <style>
                .message {
                    margin: 10px 0;
                    padding: 10px;
                    border-radius: 5px;
                }

                #usernameInput:disabled {
                    background-color: #f0f0f0;
                    cursor: not-allowed;
                }

                .message-header {
                    display: flex;
                    justify-content: space-between;
                    margin-bottom: 5px;
                }

                .message-time {
                    color: #888;
                    font-size: 0.8em;

                }
            </style>
        </head>

        <body class="p-3">
            <h3>Public Messages</h3>
            <div class="row m-1">
                <div class="col-3">
                    <label for="usernameInput" class="visually-hidden">Tên</label>
                    <input type="text" id="usernameInput" placeholder="Tên của bạn" class="form-control-plaintext px-1"
                        value="<?php echo isset($_COOKIE['uname']) ? $_COOKIE['uname'] : ''; ?>"
                        <?php echo isset($_COOKIE['uname']) ? 'disabled' : ''; ?>>
                </div>
                <div class="col-8">
                    <label for="messageInput" class="visually-hidden">Tin nhắn</label>
                    <!-- <input type="password" class="form-control" id="inputPassword2" placeholder="Password"> -->
                    <input type="text" class="form-control" id="messageInput" placeholder="Type your message for every one">
                </div>
                <div class="col-1">
                    <!-- <button type="submit" class="btn btn-primary mb-3">Post Message</button> -->
                    <button class="btn btn-primary" onclick="sendMessage()"><i class="fa-solid fa-paper-plane"></i></button>
                    <button id="clearBtn" onclick="clearMessages()" style="display:none;">Xóa hết</button>
                </div>
            </div>
            <div id="messagesContainer" style="height: 450px; overflow: auto;"></div>

            <script>
                let currentUser = null;

                async function createUser() {
                    const usernameInput = document.getElementById('usernameInput');
                    const username = usernameInput.value;
                    const response = await fetch(`?action=create_user&username=${encodeURIComponent(username)}`, {
                        method: 'POST'
                    });
                    currentUser = await response.json();

                    // Lưu username vào cookie
                    document.cookie = `uname=${encodeURIComponent(username)}; path=/; max-age=31536000`;

                    // Disable input sau khi tạo user
                    usernameInput.disabled = true;

                    // Hiển thị nút xóa nếu là admin
                    const clearBtn = document.getElementById('clearBtn');
                    clearBtn.style.display = currentUser.is_admin ? 'inline-block' : 'none';

                    return currentUser;
                }

                async function sendMessage() {
                    if (!currentUser) {
                        currentUser = await createUser();
                    }
                    const messageInput = document.getElementById('messageInput');
                    const text = messageInput.value;

                    const formData = new FormData();
                    formData.append('action', 'send_message');
                    formData.append('user_id', currentUser.id);
                    formData.append('text', text);

                    await fetch('', {
                        method: 'POST',
                        body: formData
                    });

                    messageInput.value = '';
                    loadMessages();
                }

                async function loadMessages() {
                    const response = await fetch('?action=get_messages');
                    const messages = await response.json();
                    const container = document.getElementById('messagesContainer');
                    container.innerHTML = '';

                    messages.forEach(msg => {
                        const messageDiv = document.createElement('div');
                        messageDiv.className = 'message';
                        messageDiv.style.backgroundColor = msg.color + '33';
                        messageDiv.innerHTML = `
                            <strong style="color:${msg.color}">${msg.username}:</strong>
                            <span>${msg.text}</span>
                            <span class="message-time"> <small> ${new Date(new Date(msg.timestamp + " UTC").getTime() ).toLocaleString("vi-VN")} </small> </span>
                            `;
                        container.appendChild(messageDiv);
                    });
                }

                async function clearMessages() {
                    if (!currentUser) return;

                    const response = await fetch('?action=clear_messages', {
                        method: 'POST',
                        body: new URLSearchParams({
                            'action': 'clear_messages',
                            'user_id': currentUser.id
                        })
                    });
                    const result = await response.json();

                    if (result.status === 'success') {
                        loadMessages();
                        alert('Đã xóa toàn bộ tin nhắn');
                    } else {
                        alert(result.message);
                    }
                }

                // Tự động điền username từ cookie
                document.addEventListener('DOMContentLoaded', () => {
                    const usernameInput = document.getElementById('usernameInput');
                    const username = '<?php echo isset($_COOKIE['uname']) ? $_COOKIE['uname'] : ''; ?>';

                    if (username) {
                        usernameInput.value = username;
                        usernameInput.disabled = true;
                    }

                    loadMessages();
                });
            </script>
        </body>

        </html>
<?php
        break;
}
?>